@@ -18,6 +18,7 @@ import collections
 import os
 import yaml
 from jinja2 import Template
+from copy import deepcopy
 import requests
 from cvapianalyser import CommunityEdition
 from openapispecdiff import OpenApiSpecDiff
@@ -61,6 +62,7 @@ class CloudvectorDAST(object):
         self.cv_config = None
         self.cvdast_to_auth_info = {}
         self.params_from_spec = {}
+        self.params_details_from_spec = {}
         if config_file:
             if os.path.exists(config_file):
                 self.cv_config = self._parse_cv_config(config_file)
@@ -112,6 +114,25 @@ class CloudvectorDAST(object):
         else:
             self._process_param_diff(apis_to_check, False)
 
+        # params_captured_in_traffic_new = deepcopy(self.params_captured_in_traffic)
+        # for k,v in self.params_captured_in_traffic.items():
+        #     if type(v) is list:
+        #         for each in v:
+        #             for i, j in self._recursive_items(each):
+        #                 if i not in self.params_captured_in_traffic:
+        #                     params_captured_in_traffic_new[i] = j
+        #     else:
+        #         for i, j in self._recursive_items(v):
+        #             if i not in self.params_captured_in_traffic:
+        #                 params_captured_in_traffic_new[i] = j
+        #
+        # print("\n\n\n\n")
+        # print(params_captured_in_traffic_new)
+        # print("\n\n\n\n")
+        # print(self.params_captured_in_traffic)
+        # print("\n\n\n\n")
+
+        #self.params_captured_in_traffic = params_captured_in_traffic_new
         with open("tests/params_captured.json", "w+") as fobj:
             json.dump(self.params_captured_in_traffic, fobj)
         self.create_pyfixtures()
@@ -140,6 +161,13 @@ class CloudvectorDAST(object):
                 input_vars[str(key).strip()] = value
         return input_vars
 
+    def _recursive_items(self, d):
+        for key, value in d.items():
+            if type(value) is dict:
+                yield (key, value)
+                yield from self._recursive_items(value)
+            else:
+                yield (key, value)
     # def _update_params_captured_for_auth(self, auth_info):
     #     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "tests/authcreds.json"),"w+") as fobj:
     #         json.dumps(auth_info)
@@ -166,6 +194,7 @@ class CloudvectorDAST(object):
                 x["host"] = host
                 x["header"] = {}
                 x["params"] = []
+                x["detailed_params"] = {}
                 x["body"] = {}
                 x["rsp_body"] = {}
                 x["url"] = ""
@@ -192,7 +221,14 @@ class CloudvectorDAST(object):
                     else:
                         x["params"].append(_["name"])
                         x["body"][_["name"]] = ""
+                    if _.get("value"):
+                        if type(_.get("value")) is list:
+                            for e in _.get("value"):
+                                x["detailed_params"].update(e)
+                        else:
+                            x["detailed_params"].update(_.get("value"))
                 self.params_from_spec[api] = {_:[] for _ in x["params"]}
+                self.params_details_from_spec.update(x["detailed_params"])
                 # if str(self.cv_config["authentication"]["api"]).lower() in str(x["url"]).lower():
                 #     self.cvdast_to_auth_info = x
                 apis_from_spec.append([x, None])
@@ -296,8 +332,8 @@ class CloudvectorDAST(object):
         print("\n\ncreating Pytest fixtures....\n")
         with open(os.path.join(templates_dir, 'conftest.j2')) as file_:
             template = Template(file_.read())
-        code = template.render(api_info=self.params_captured_in_traffic)
-        print(self.validate_pycode_for_syntax(code))
+        #code = template.render(api_info=self.params_captured_in_traffic)
+        #print(self.validate_pycode_for_syntax(code))
         if not os.path.exists("tests"):
             os.mkdir("tests")
         auth_api = None
@@ -315,8 +351,6 @@ class CloudvectorDAST(object):
                     auth_inputs.append(template_found.group(1))
                     auth_api_headers[k] = auth_inputs[-1]
             for k, v in auth_api_payload.items():
-                print(k)
-                print(v)
                 template_found = re.search(r"\[\'([A-Za-z0-9_]+)\'\]", str(v))
                 if template_found:
                     auth_inputs.append(template_found.group(1))
@@ -328,8 +362,20 @@ class CloudvectorDAST(object):
             for k, v in params_info.items():
                 if k not in self.params_captured_in_traffic[api]:
                     self.params_captured_in_traffic[api][k]=v
+        nested_params={}
+        more_params = []
+        for param, info in self.params_details_from_spec.items():
+            tmp = []
+            if type(info) is list:
+                info = info[0]
+            for k, v in self._recursive_items(info):
+                if type(v) is not dict:
+                    tmp.append(k)
+            nested_params[param] = tmp
+            more_params+=tmp
         with open("tests/conftest.py", 'w+') as fh:
-            fh.write(template.render(api_info=self.params_captured_in_traffic,AUTH_API=auth_api,
+            fh.write(template.render(api_info=self.params_captured_in_traffic, api_detailed_info=self.params_details_from_spec,
+                                     nested_params=nested_params, AUTH_API=auth_api, more_params=more_params,
                                      AUTH_API_HEADERS=auth_api_headers,AUTH_API_PAYLOAD=json.dumps(auth_api_payload),
                                      AUTH_RESP_KEY=auth_api_response_key, AUTH_INPUTS=auth_inputs))
         print("\n\t\t......done creating pytest fixtures (conftest.py)")
@@ -385,15 +431,17 @@ class CloudvectorDAST(object):
                 })
         self.assertions_map[url].update(assertions_map)
 
-    def _create_custom_validations(self, params_to_validate, actual_params):
+    def _create_custom_validations(self, params_to_validate, info):
         validations = []
-        for param in actual_params:
-            if param in params_to_validate:
-                for condition, to_check in params_to_validate[param].items():
-                    if condition == "missing":
-                        validations.append([str(param) + "_missing", str(param) + " = ''", to_check])
-                    elif condition == "invalid":
-                        validations.append([str(param) + "_invalid", str(param) + " = 'iamdummy'", to_check])
+        for method, minfo in info.items():
+            actual_params = minfo.get("params")
+            for param in actual_params:
+                if param in params_to_validate:
+                    for condition, to_check in params_to_validate[param].items():
+                        if condition == "missing":
+                            validations.append([str(param) + "_missing", str(param) + " = ''", to_check])
+                        elif condition == "invalid":
+                            validations.append([str(param) + "_invalid", str(param) + " = 'iamdummy'", to_check])
         return validations
 
     def create_pytest_methods(self, cv_events, custom_validations):
